﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using KendoUI.Models;
using System.Web.Script.Serialization;


namespace KendoUI.Controllers
{
    public class HomeController : Controller
    {
        public ActionResult Index()
        {
            return View("IndexInlineEditing");
        }

        public ActionResult Edit(string id)
        {
            KendoUISampleEntities db = new KendoUISampleEntities();
            return View(db.Contacts.First(res => res.FirstName.Contains(id)));
        }

        [HttpPost]
        public ActionResult Edit(KendoUI.Contact person)
        {
            KendoUISampleEntities db = new KendoUISampleEntities();
            var personData = db.Contacts.First(res => res.FirstName.Contains(person.FirstName));
            personData.LastName = person.LastName;
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        [OutputCache(Duration=0)]
        public JsonResult IndexJson()
        {
            KendoUISampleEntities db = new KendoUISampleEntities();
            IEnumerable<ContactModel> person = (from e in db.Contacts
                                               orderby e.LastName, e.FirstName
                                                select new ContactModel
                                               {
                                                   ID = e.ContactID,
                                                   FirstName = e.FirstName,
                                                   LastName = e.LastName
                                               });
            return Json(person, JsonRequestBehavior.AllowGet);
        }

        public ActionResult IndexInlineEditing()
        {
            return View();
        }


        /// <summary>
        /// Create Operation
        /// </summary>
        /// <param name="models"></param>
        public JsonResult Create(string models)
        {
            KendoUISampleEntities db = new KendoUISampleEntities();
            var contacts = new JavaScriptSerializer().Deserialize<IList<ContactModel>>(models);
            foreach (var contact in contacts)
            {
                db.AddToContacts(new Contact() { FirstName = contact.FirstName, LastName = contact.LastName });
            }
            db.SaveChanges();
            return Json(models, JsonRequestBehavior.AllowGet);
        }

        /// <summary>
        /// Update Operation
        /// </summary>
        /// <param name="models"></param>
        public JsonResult Update(string models)
        {
            KendoUISampleEntities db = new KendoUISampleEntities();
            var contacts = new JavaScriptSerializer().Deserialize<IList<ContactModel>>(models);
            foreach (var contact in contacts)
            {
                var _contact = db.Contacts.Single(res => res.ContactID == contact.ID);
                _contact.FirstName = contact.FirstName;
                _contact.LastName = contact.LastName;
                db.SaveChanges();
            }
            return Json(models, JsonRequestBehavior.AllowGet);
        }

        /// <summary>
        /// Delete Operation
        /// </summary>
        /// <param name="models"></param>
        public JsonResult Destroy(string models)
        {
            KendoUISampleEntities db = new KendoUISampleEntities();
            var contactstodelete = new JavaScriptSerializer().Deserialize<IList<ContactModel>>(models);
            foreach (var contact in contactstodelete)
            {
                Contact _contact = db.Contacts.Single(res => res.ContactID == contact.ID);
                db.Contacts.DeleteObject(_contact);
                db.SaveChanges();
            }
            return Json(models, JsonRequestBehavior.AllowGet);
        }
    }
}
